@@ -211,17 +211,17 @@ bool WPG1Parser::parse()
 				
 		WPG_DEBUG_MSG(("\n"));
 		if(index < 0)
-			WPG_DEBUG_MSG(("Unknown record type 0x%02x at %d  size %d\n", 
+			WPG_DEBUG_MSG(("Unknown record type 0x%02x at %li  size %d\n", 
 				recordType, recordPos, m_recordLength));
 		else
 		{
 			Method recordHandler = handlers[index].handler;
 			if(!recordHandler)
-				WPG_DEBUG_MSG(("Record '%s' (ignored) type 0x%02x at %d  size %d\n", 
+				WPG_DEBUG_MSG(("Record '%s' (ignored) type 0x%02x at %li  size %d\n", 
 					handlers[index].name, recordType, recordPos, m_recordLength));
 			else
 			{
-				WPG_DEBUG_MSG(("Record '%s' type 0x%02x at %d  size %d\n", 
+				WPG_DEBUG_MSG(("Record '%s' type 0x%02x at %li  size %d\n", 
 					handlers[index].name, recordType, recordPos, m_recordLength));
 					
 				// invoke the handler for this record
@@ -232,7 +232,7 @@ bool WPG1Parser::parse()
 		//if(m_input->tell() > m_recordEnd+1)
 		{
 			//WPG_DEBUG_MSG(("Record 0x%x consumes more bytes than necessary!\n", recordType));
-			WPG_DEBUG_MSG(("Current stream position: %d\n", m_input->tell()));
+			WPG_DEBUG_MSG(("Current stream position: %li\n", m_input->tell()));
 		}
 	
 		if(m_exit) break;
@@ -453,20 +453,24 @@ void WPG1Parser::handleEllipse()
 // NOTE: delete the buffer by yourself!
 unsigned char* WPG1Parser::decodeRLE(int width, int height, int depth)
 {
+	if (depth <= 0 || width <= 0 || height <= 0)
+		return 0;
+
 	// round to the next byte
-	int scanline_width = (width * depth + 7)/8;
-	int buffer_size = scanline_width * height;
+	unsigned scanline_width = (width * depth + 7)/8;
+	unsigned buffer_size = scanline_width * height;
 	WPG_DEBUG_MSG(("Scanline width: %d\n", scanline_width));
 	WPG_DEBUG_MSG(("Output size: %d\n", buffer_size));
 
 	// buffer for the result
 	unsigned char* buffer = new unsigned char[buffer_size];
+	
+	if (!buffer)
+		return 0;
 
 	// pointer to write decoded data
 	unsigned char* ptr = buffer;
 
-	// Decode the RLE bitmap buffer
-	// FIXME check for ptr, it should not go out of bound!!
 	WPG_DEBUG_MSG(("Decoding RLE data\n"));
 	for(; m_input->tell() < m_recordEnd; )
 	{
@@ -479,7 +483,7 @@ unsigned char* WPG1Parser::decodeRLE(int width, int height, int depth)
 			unsigned char pixel = (count > 0) ? readU8() : 0xff;
 			if(count == 0)
 				count = (int)readU8();
-			for( ; count; --count)
+			for( ; count && ptr < buffer + buffer_size; --count)
 				*ptr++ = pixel;
 		}
 		else
@@ -488,7 +492,7 @@ unsigned char* WPG1Parser::decodeRLE(int width, int height, int depth)
 			if(count > 0)
 			{
 				// literal byte copy
-				for( ; count; --count)
+				for( ; count && ptr < buffer + buffer_size; --count)
 					*ptr++ = readU8();
 			}
 			else
@@ -496,8 +500,10 @@ unsigned char* WPG1Parser::decodeRLE(int width, int height, int depth)
 				// copy entire scan line
 				count = (int)readU8();
 				unsigned char* raster_source = ptr - scanline_width;
+				if (!raster_source || raster_source < buffer)
+					return 0;
 				for( ; count; --count)
-					for(int r = 0; r < scanline_width; r++)
+					for(unsigned r = 0; r < scanline_width && ptr < buffer + buffer_size; r++)
 						*ptr++ = raster_source[r];
 			}
 		}
@@ -515,7 +521,7 @@ int width, int height, int depth)
 		return;
 
 	// round to the next byte
-	int scanline_width = (width * depth + 7)/8;
+	unsigned scanline_width = (width * depth + 7)/8;
 
 	// 1-bit image: black and white
 	if(depth == 1)
@@ -582,6 +588,12 @@ void WPG1Parser::handleBitmapTypeOne()
 		hres = 1200;
 	if(vres <= 0)
 		vres = 1200;
+	if (width < 0)
+		width = 0;
+	if (height < 0)
+		height = 0;
+	if (depth < 0)
+		depth = 0;
 
 	// Bitmap Type 1 does not specify position
 	// Assume on the corner (0,0)
@@ -592,10 +604,13 @@ void WPG1Parser::handleBitmapTypeOne()
 	bitmap.rect.y2 = (double)height/(double)vres;
 
 	unsigned char* buffer = decodeRLE(width, height, depth);
-	fillPixels(bitmap, buffer, width, height, depth);
-	delete [] buffer;
+	if (buffer)
+	{
+		fillPixels(bitmap, buffer, width, height, depth);
+		delete [] buffer;
 
-	m_painter->drawBitmap(bitmap);
+		m_painter->drawBitmap(bitmap);
+	}
 }
 
 void WPG1Parser::handleBitmapTypeTwo()
@@ -632,6 +647,12 @@ void WPG1Parser::handleBitmapTypeTwo()
 		hres = 1200;
 	if(vres <= 0)
 		vres = 1200;
+	if (width < 0)
+		width = 0;
+	if (height < 0)
+		height = 0;
+	if (depth < 0)
+		depth = 0;
 
 	y1 = m_height - y1;
 	y2 = m_height - y2;
@@ -640,7 +661,7 @@ void WPG1Parser::handleBitmapTypeTwo()
 	long xs2 = (x1 <= x2) ? x2 : x1;
 	long ys1 = (y1 <= y2) ? y1 : y2;
 	long ys2 = (y1 <= y2) ? y2 : y1;
-	WPG_DEBUG_MSG(("%d %d   %d %d\n", xs1, ys1, xs2, ys2));
+	WPG_DEBUG_MSG(("%li %li   %li %li\n", xs1, ys1, xs2, ys2));
 
 	libwpg::WPGBitmap bitmap(width, height);
 	bitmap.rect.x1 = (double)xs1/1200.0;
@@ -649,10 +670,13 @@ void WPG1Parser::handleBitmapTypeTwo()
 	bitmap.rect.y2 = (double)(ys2)/1200.0;
 
 	unsigned char* buffer = decodeRLE(width, height, depth);
-	fillPixels(bitmap, buffer, width, height, depth);
-	delete [] buffer;
+	if (buffer)
+	{
+		fillPixels(bitmap, buffer, width, height, depth);
+		delete [] buffer;
 
-	m_painter->drawBitmap(bitmap);
+		m_painter->drawBitmap(bitmap);
+	}
 }
 
 void WPG1Parser::resetPalette()
