@@ -225,7 +225,9 @@ WPG2Parser::WPG2Parser(WPXInputStream *input, libwpg::WPGPaintInterface* painter
 	m_compoundFramed(false),
 	m_compoundClosed(false),
 	m_bitmap(),
-	m_binaryData()
+	m_binaryData(),
+	m_hFlipped(false),
+	m_vFlipped(false)
 {
 }
 
@@ -1351,6 +1353,14 @@ void WPG2Parser::handleBitmap()
 		return;
 	ObjectCharacterization objCh;
 	parseCharacterization(&objCh);
+	
+	if (objCh.scale)
+	{
+		if (objCh.sxcos < 0)
+			m_hFlipped = true;
+		if (objCh.sycos < 0)
+			m_vFlipped = true;
+	}
 	m_matrix = objCh.matrix;
 
 	long x1 = (m_doublePrecision) ? readS32() : readS16();
@@ -1387,9 +1397,9 @@ void WPG2Parser::handleBitmapData()
 {
 	if (!m_graphicsStarted)
 		return;
-	unsigned long width = (m_doublePrecision) ? readU32() : readU16();
-	unsigned long height = (m_doublePrecision) ? readU32() : readU16();
-	unsigned color_format = (readU8());
+	unsigned long width = readU16();
+	unsigned long height = readU16();
+	unsigned color_format = readU8();
 	unsigned compression_format = readU8();
 
 	WPG_DEBUG_MSG(("     dimension : %g, %g  %g, %g\n", m_bitmap.x1, m_bitmap.y1, m_bitmap.x2, m_bitmap.y2));
@@ -1435,6 +1445,7 @@ void WPG2Parser::handleBitmapData()
 	
 	// prepare the bitmap actual data
 	unsigned char* buffer = 0;
+	unsigned char* ptr = 0;
 
 	// plain data, uncompression
 	if(compression_format==0)
@@ -1446,12 +1457,12 @@ void WPG2Parser::handleBitmapData()
 	// run-length encoding
 	else if(compression_format==1)
 	{
-		unsigned char * ptr = buffer;
 		unsigned char data[256];
 		bool xor_raster = false;
 		unsigned char* next_scanline = buffer;
-		int data_size = 1;
+		unsigned data_size = 1;
 		buffer = new unsigned char[tmpBufferSize];
+		ptr = buffer;
 
 		WPG_DEBUG_MSG(("Decoding RLE data\n"));
 
@@ -1462,7 +1473,7 @@ void WPG2Parser::handleBitmapData()
 			// specify data size
 			if(opcode == 0x7d)
 			{
-				int new_data_size = (int)readU8();
+				unsigned new_data_size = readU8();
 				if(new_data_size != data_size)
 				{
 					data_size = new_data_size;
@@ -1482,9 +1493,9 @@ void WPG2Parser::handleBitmapData()
 			// a run of black (index #0)
 			else if(opcode == 0x7f)
 			{
-				int count = 1 + (int)readU8();
+				unsigned count = 1 + readU8();
 				for( ; count ; --count )
-					for(int j = 0; j < data_size; j++)
+					for(unsigned j = 0; j < data_size; j++)
 						*ptr++ =  0;
 				
 			}
@@ -1492,26 +1503,26 @@ void WPG2Parser::handleBitmapData()
 			// a run of white (index #255)
 			else if(opcode == 0xff)
 			{
-				int count = 1 + (int)readU8();
+				unsigned count = 1 + readU8();
 
 				for( ; count ; --count )
-					for(int j = 0; j < data_size; j++)
+					for(unsigned j = 0; j < data_size; j++)
 						*ptr++ =  255;
 			}
 
 			// extend previous run
 			else if(opcode == 0xfd)
 			{
-				int count = 1 + (int)readU8();
+				unsigned count = 1 + readU8();
 				for( ; count; --count)
-					for(int j = 0; j < data_size; j++)
+					for(unsigned j = 0; j < data_size; j++)
 						*ptr++ = data[j];
 			}
 
 			// repeat raster
 			else if(opcode == 0xfe)
 			{
-				int count = 1 + (int)readU8();
+				unsigned count = 1 + readU8();
 				unsigned char* raster_source = ptr - raster_len;
 				for( ; count; --count)
 					for(unsigned long r = 0; r < raster_len; r++)
@@ -1524,7 +1535,6 @@ void WPG2Parser::handleBitmapData()
 				// Xor-ing will happen when about to enter next raster
 				// see after the last if down below
 				xor_raster = true;
-				raster_len = data_size * width;
 				next_scanline = ptr + raster_len;
 			}
 
@@ -1533,20 +1543,20 @@ void WPG2Parser::handleBitmapData()
 			// a run of data
 			else if(opcode >= 0x80)
 			{
-				int count = 1 + (opcode & 0x7f);
-				for(int i = 0; i < data_size; i++)
+				unsigned count = 1 + (opcode & 0x7f);
+				for(unsigned i = 0; i < data_size; i++)
 					data[i] = readU8();
 				for( ; count; --count)
-					for(int j = 0; j < data_size; j++)
+					for(unsigned j = 0; j < data_size; j++)
 						*ptr++ = data[j];
 			}
 
 			// simple copy
 			else if(opcode <= 0x7f)
 			{
-				int count = opcode + 1;
+				unsigned count = opcode + 1;
 				for( ; count; --count)
-					for(int j = 0; j < data_size; j++)
+					for(unsigned j = 0; j < data_size; j++)
 						*ptr++ = readU8();
 			}
 
@@ -1564,7 +1574,6 @@ void WPG2Parser::handleBitmapData()
 				xor_raster = false;
 
 				// current raster must be XORed with previous raster
-				raster_len = data_size * width;
 				unsigned char* current = next_scanline - raster_len;
 				unsigned char* previous = current - raster_len;
 				for( unsigned long r = 0; r < raster_len; r++)
@@ -1581,7 +1590,7 @@ void WPG2Parser::handleBitmapData()
 	if(!buffer) return;
 
 	// prepare the bitmap structure for the listener
-	libwpg::WPGBitmap bitmap(width, height);
+	libwpg::WPGBitmap bitmap(width, height, m_vFlipped, m_hFlipped);
 	bitmap.rect.x1 = m_bitmap.x1;
 	bitmap.rect.y1 = m_bitmap.y1;
 	bitmap.rect.x2 = m_bitmap.x2;
