@@ -32,9 +32,11 @@ class libwpg::WPGBitmap::Private
 public:
 	int width;
 	int height;
+	bool vFlip;
+	bool hFlip;
 	WPGColor* pixels;
 
-	Private(int w, int h): width(w), height(h), pixels(0) {}
+	Private(int w, int h): width(w), height(h), vFlip(false), hFlip(false), pixels(0) {}
 };
 
 libwpg::WPGBitmap::WPGBitmap(int width, int height): rect(), d(new Private(width, height))
@@ -42,6 +44,15 @@ libwpg::WPGBitmap::WPGBitmap(int width, int height): rect(), d(new Private(width
 	d->pixels = new WPGColor[width*height];
 }
 
+libwpg::WPGBitmap::WPGBitmap(int width, int height, bool verticalFlip, bool horizontalFlip) :
+	rect(),
+	d(new Private(width, height))
+{
+	d->vFlip = verticalFlip;
+	d->hFlip = horizontalFlip;
+	d->pixels = new WPGColor[width*height];
+}
+
 libwpg::WPGBitmap::~WPGBitmap()
 {
 	delete [] d->pixels;
@@ -98,7 +109,7 @@ void libwpg::WPGBitmap::setPixel(int x, int y, const libwpg::WPGColor& color)
 
 void libwpg::WPGBitmap::generateBase64DIB(libwpg::WPGString& bmp) const
 {
-	if (d->height < 0 || d->width < 0)
+	if (d->height <= 0 || d->width <= 0)
 		return;
 
 	unsigned tmpPixelSize = (unsigned)(d->height * d->width);
@@ -142,16 +153,48 @@ void libwpg::WPGBitmap::generateBase64DIB(libwpg::WPGString& bmp) const
 	WPG_DEBUG_MSG(("WPGBitmap: DIB info header end = %i\n", tmpBufferPosition - 1));
 
 	// Write DIB Image data
-	for (int i = d->height - 1; i >= 0 && tmpBufferPosition < tmpDIBFileSize; i--)
-	{
-		for (int j = 0; j < d->width && tmpBufferPosition < tmpDIBFileSize; j++)
+	
+	int i = 0; int j = 0;
+	if (d->vFlip)
+		for	(i = 0; i < d->height && tmpBufferPosition < tmpDIBFileSize; i++)
 		{
-			writeU8(tmpDIBBuffer, tmpBufferPosition, d->pixels[i*d->width + j].blue);
-			writeU8(tmpDIBBuffer, tmpBufferPosition, d->pixels[i*d->width + j].green);
-			writeU8(tmpDIBBuffer, tmpBufferPosition, d->pixels[i*d->width + j].red);
-			writeU8(tmpDIBBuffer, tmpBufferPosition, d->pixels[i*d->width + j].alpha);
+			if (d->hFlip)
+				for (j = d->width - 1; j >= 0 && tmpBufferPosition < tmpDIBFileSize; j--)
+				{
+					writeU8(tmpDIBBuffer, tmpBufferPosition, d->pixels[i*d->width + j].blue);
+					writeU8(tmpDIBBuffer, tmpBufferPosition, d->pixels[i*d->width + j].green);
+					writeU8(tmpDIBBuffer, tmpBufferPosition, d->pixels[i*d->width + j].red);
+					writeU8(tmpDIBBuffer, tmpBufferPosition, d->pixels[i*d->width + j].alpha);
+				}
+			else
+				for (int j = 0; j < d->width && tmpBufferPosition < tmpDIBFileSize; j++)
+				{
+					writeU8(tmpDIBBuffer, tmpBufferPosition, d->pixels[i*d->width + j].blue);
+					writeU8(tmpDIBBuffer, tmpBufferPosition, d->pixels[i*d->width + j].green);
+					writeU8(tmpDIBBuffer, tmpBufferPosition, d->pixels[i*d->width + j].red);
+					writeU8(tmpDIBBuffer, tmpBufferPosition, d->pixels[i*d->width + j].alpha);
+				}
+		}
+	else
+		for (int i = d->height - 1; i >= 0 && tmpBufferPosition < tmpDIBFileSize; i--)
+		{
+			if (d->hFlip)
+				for (j = d->width - 1; j >= 0 && tmpBufferPosition < tmpDIBFileSize; j--)
+				{
+					writeU8(tmpDIBBuffer, tmpBufferPosition, d->pixels[i*d->width + j].blue);
+					writeU8(tmpDIBBuffer, tmpBufferPosition, d->pixels[i*d->width + j].green);
+					writeU8(tmpDIBBuffer, tmpBufferPosition, d->pixels[i*d->width + j].red);
+					writeU8(tmpDIBBuffer, tmpBufferPosition, d->pixels[i*d->width + j].alpha);
+				}
+			else
+				for (int j = 0; j < d->width && tmpBufferPosition < tmpDIBFileSize; j++)
+				{
+					writeU8(tmpDIBBuffer, tmpBufferPosition, d->pixels[i*d->width + j].blue);
+					writeU8(tmpDIBBuffer, tmpBufferPosition, d->pixels[i*d->width + j].green);
+					writeU8(tmpDIBBuffer, tmpBufferPosition, d->pixels[i*d->width + j].red);
+					writeU8(tmpDIBBuffer, tmpBufferPosition, d->pixels[i*d->width + j].alpha);
+				}
 		}
-	}
 
 	WPG_DEBUG_MSG(("WPGBitmap: DIB file size = %i\n", tmpBufferPosition - 1));
 
