@@ -140,7 +140,8 @@ const uint8_t *WPGFileStream::read(size_t numBytes, size_t &numBytesRead)
 	// hmm, we cannot: go back by the bytes we read ahead && invalidate the buffer
 	if (d->readBuffer)
 	{
-		d->file.seekg(d->readBufferPos - d->readBufferLength, std::ios::cur);
+		d->file.seekg((unsigned long)d->file.tellg() - d->readBufferLength, std::ios::beg);
+		d->file.seekg(d->readBufferPos, std::ios::cur);
 		delete [] d->readBuffer;
 		d->readBuffer = 0; d->readBufferPos = 0; d->readBufferLength = 0;
 	}
@@ -185,7 +186,7 @@ const uint8_t *WPGFileStream::read(size_t numBytes, size_t &numBytesRead)
 
 long WPGFileStream::tell()
 {
-	return d->file.good() ? (long)d->file.tellg() : -1L;
+	return d->file.good() ? (long)((long)d->file.tellg() - d->readBufferLength + d->readBufferPos) : -1L;
 }
 
 int WPGFileStream::seek(long offset, WPX_SEEK_TYPE seekType)
@@ -206,7 +207,8 @@ int WPGFileStream::seek(long offset, WPX_SEEK_TYPE seekType)
 	
 	if (d->readBuffer) // seeking outside of the buffer, so invalidate the buffer
 	{
-		d->file.seekg(d->readBufferPos - d->readBufferLength, std::ios::cur);
+		d->file.seekg((unsigned long)d->file.tellg() - d->readBufferLength, std::ios::beg);
+		d->file.seekg(d->readBufferPos, std::ios::cur);
 		delete [] d->readBuffer;
 		d->readBuffer = 0; d->readBufferPos = 0; d->readBufferLength = 0;
 	}
@@ -222,7 +224,7 @@ int WPGFileStream::seek(long offset, WPX_SEEK_TYPE seekType)
 
 bool WPGFileStream::atEOS()
 {
-	return (d->file.tellg() >= (long)d->streamSize);
+	return (tell() >= (long)d->streamSize);
 }
 
 bool WPGFileStream::isOLEStream()
@@ -232,7 +234,8 @@ bool WPGFileStream::isOLEStream()
 
 	if (d->readBuffer)
 	{
-		d->file.seekg(d->readBufferPos - d->readBufferLength, std::ios::cur);
+		d->file.seekg((unsigned long)d->file.tellg() - d->readBufferLength, std::ios::beg);
+		d->file.seekg(d->readBufferPos, std::ios::cur);
 		delete [] d->readBuffer;
 		d->readBuffer = 0; d->readBufferPos = 0; d->readBufferLength = 0;
 	}
@@ -240,12 +243,9 @@ bool WPGFileStream::isOLEStream()
 	if (d->buffer.str().empty())
 		d->buffer << d->file.rdbuf();
 	Storage tmpStorage( d->buffer );
-	if (tmpStorage.isOLEStream())
-	{
-		seek(0, WPX_SEEK_SET);
-		return true;
-	}
 	seek(0, WPX_SEEK_SET);
+	if (tmpStorage.isOLEStream())
+		return true;
 	return false;
 }
 
@@ -256,7 +256,8 @@ bool WPGFileStream::isOLEStream()
 
 	if (d->readBuffer)
 	{
-		d->file.seekg(d->readBufferPos - d->readBufferLength, std::ios::cur);
+		d->file.seekg((unsigned long)d->file.tellg() - d->readBufferLength, std::ios::beg);
+		d->file.seekg(d->readBufferPos, std::ios::cur);
 		delete [] d->readBuffer;
 		d->readBuffer = 0; d->readBufferPos = 0; d->readBufferLength = 0;
 	}
